@@ -1,8 +1,46 @@
+from django.conf import settings
 from django.core.exceptions import ImproperlyConfigured
 
 import redis
 from rq.queue import FailedQueue, Queue
 
+import thread_queue
+
+
+def get_commit_mode():
+    """
+    Disabling AUTOCOMMIT causes enqueued jobs to be stored in a temporary queue.
+    Jobs in this queue are only enqueued after the request is completed and are
+    discarded if the request causes an exception (similar to db transactions).
+
+    To disable autocommit, put this in settings.py:
+    RQ = {
+        'AUTOCOMMIT': False,
+    }
+    """
+    RQ = getattr(settings, 'RQ', {})
+    return RQ.get('AUTOCOMMIT', True)
+
+
+class DjangoRQ(Queue):
+    """
+    A subclass of RQ's QUEUE that allows jobs to be stored temporarily to be
+    enqueued later at the end of Django's request/response cycle.
+    """
+
+    def __init__(self, *args, **kwargs):
+        self._autocommit = get_commit_mode()
+        return super(DjangoRQ, self).__init__(*args, **kwargs)
+
+    def original_enqueue(self, *args, **kwargs):
+        return super(DjangoRQ, self).enqueue(*args, **kwargs)
+
+    def enqueue(self, f, *args, **kwargs):
+        if self._autocommit:
+            return self.original_enqueue(f, *args, **kwargs)
+        else:
+            thread_queue.add(self, f, args, kwargs)
+
 
 def get_redis_connection(config):
     """
@@ -10,9 +48,26 @@ def get_redis_connection(config):
     """
     if 'URL' in config:
         return redis.from_url(config['URL'], db=config['DB'])
+    if 'USE_REDIS_CACHE' in config.keys():
+
+        from django.core.cache import get_cache
+        cache = get_cache(config['USE_REDIS_CACHE'])
+
+        if hasattr(cache, 'client'):
+            # We're using django-redis. The cache's `client` attribute
+            # is a pluggable backend that return its Redis connection as
+            # its `client`
+            try:
+                return cache.client.client
+            except NotImplementedError:
+                pass
+        else:
+            # We're using django-redis-cache
+            return cache._client
+
     return redis.Redis(host=config['HOST'],
-        port=config['PORT'], db=config['DB'],
-        password=config.get('PASSWORD', None))
+                       port=config['PORT'], db=config['DB'],
+                       password=config.get('PASSWORD', None))
 
 
 def get_connection(name='default'):
@@ -31,24 +86,39 @@ def get_connection_by_index(index):
     return get_redis_connection(QUEUES_LIST[index]['connection_config'])
 
 
-def get_queue(name='default', default_timeout=None, async=True):
+def get_queue(name='default', default_timeout=None, async=None):
     """
     Returns an rq Queue using parameters defined in ``RQ_QUEUES``
     """
-    return Queue(name, default_timeout=default_timeout,
-                 connection=get_connection(name), async=async)
+    from .settings import QUEUES
+
+    # If async is provided, use it, otherwise, get it from the configuration
+    if async is None:
+        async = QUEUES[name].get('ASYNC', True)
+
+    return DjangoRQ(name, default_timeout=default_timeout,
+                    connection=get_connection(name), async=async)
 
 
 def get_queue_by_index(index):
     """
     Returns an rq Queue using parameters defined in ``QUEUES_LIST``
     """
-    from .settings import QUEUES_LIST    
+    from .settings import QUEUES_LIST
     config = QUEUES_LIST[int(index)]
     if config['name'] == 'failed':
-        return FailedQueue(connection=get_redis_connection(config['connection_config']))    
-    return Queue(config['name'],
-                 connection=get_redis_connection(config['connection_config']))
+        return FailedQueue(connection=get_redis_connection(config['connection_config']))
+    return DjangoRQ(
+        config['name'],
+        connection=get_redis_connection(config['connection_config']),
+        async=config.get('ASYNC', True))
+
+
+def get_failed_queue(name='default'):
+    """
+    Returns the rq failed Queue using parameters defined in ``RQ_QUEUES``
+    """
+    return FailedQueue(connection=get_connection(name))
 
 
 def get_queues(*queue_names):
